@@ -9,21 +9,38 @@ import sys
 import typing
 import warnings
 import setuptools.msvc
-import distutils.util
 import pefile
 import machomachomangler.pe
 from . import dll_list
 
 
+pefile.fast_load = True
+
+
 class PEContext:
     """Context manager for PE file."""
-    def __init__(self, name: str) -> None:
-        self._pe = pefile.PE(name)
+    def __init__(self, path: str, parse_imports: bool, verbose: int) -> None:
+        """
+        path: path to PE file
+        parse_imports: whether to parse the import table and delay import table
+        verbose: verbosity level
+        """
+        self._pe = pefile.PE(path)
+        if parse_imports:
+            self._pe.parse_data_directories([
+                pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_IMPORT'],
+                pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT'],
+            ], import_dllnames_only=True)
+        self._name = os.path.basename(path)
+        self._verbose = verbose
 
     def __enter__(self) -> pefile.PE:
         return self._pe
 
     def __exit__(self, exc_type, exc_value, traceback):
+        if self._verbose >= 2:
+            for w in self._pe.get_warnings():
+                print(f'{self._name}: {w}')
         self._pe.close()
 
 
@@ -50,7 +67,7 @@ def _translate_directory() -> typing.Callable[[str, int], str]:
     """Closure that computes certain values once only for determining how to
     translate a directory when searching for DLLs on Windows.
 
-    Returns a function translate_directory(directory: int, bitness: str) -> str
+    Returns a function translate_directory(directory: str, bitness: int) -> str
     that performs directory translations. Given a directory to search for a DLL
     of the given bitness, translate the directory, taking the Windows file
     system redirector into account.
@@ -94,7 +111,7 @@ def _translate_directory() -> typing.Callable[[str, int], str]:
     warned = False
 
     if interpreter_bitness == os_bitness == 64:
-        def translate_directory(directory: int, bitness: str) -> str:
+        def translate_directory(directory: str, bitness: int) -> str:
             if bitness == 64:
                 return directory
             # perform file system redirection manually
@@ -108,7 +125,7 @@ def _translate_directory() -> typing.Callable[[str, int], str]:
             return directory
         return translate_directory
     elif interpreter_bitness == 32 and os_bitness == 64:
-        def translate_directory(directory: int, bitness: str) -> str:
+        def translate_directory(directory: str, bitness: int) -> str:
             if bitness == 32:
                 return directory
             # disable file system redirection
@@ -178,12 +195,12 @@ def find_library(name: str, wheel_dirs: typing.Optional[typing.Iterable], bitnes
     return None
 
 
-def get_direct_needed(lib_path: str, include_delay_imports: bool = True) -> set:
+def get_direct_needed(lib_path: str, include_delay_imports: bool, verbose: int) -> set:
     """Given the path to a shared library, return a set containing the lowercase
     DLL names of all its direct dependencies.
     If include_delay_imports is True, delay-loaded dependencies are included.
     Otherwise, they are not included"""
-    with PEContext(lib_path) as pe:
+    with PEContext(lib_path, True, verbose) as pe:
         imports = []
         if include_delay_imports:
             attrs = ('DIRECTORY_ENTRY_IMPORT', 'DIRECTORY_ENTRY_DELAY_IMPORT')
@@ -198,7 +215,7 @@ def get_direct_needed(lib_path: str, include_delay_imports: bool = True) -> set:
     return needed
 
 
-def get_direct_mangleable_needed(lib_path: str, no_dlls: set, no_mangles: set) -> set:
+def get_direct_mangleable_needed(lib_path: str, no_dlls: set, no_mangles: set, verbose: int) -> set:
     """Given the path to a shared library, return a set containing the lowercase
     DLL names of all direct dependencies that belong in the wheel and should be
     name-mangled.
@@ -207,7 +224,7 @@ def get_direct_mangleable_needed(lib_path: str, no_dlls: set, no_mangles: set) -
     wheel.
 
     no_mangles is a set of lowercase additional DLL names not to mangle."""
-    with PEContext(lib_path) as pe:
+    with PEContext(lib_path, True, verbose) as pe:
         imports = []
         for attr in ('DIRECTORY_ENTRY_IMPORT', 'DIRECTORY_ENTRY_DELAY_IMPORT'):
             if hasattr(pe, attr):
@@ -229,7 +246,8 @@ def get_direct_mangleable_needed(lib_path: str, no_dlls: set, no_mangles: set) -
 def get_all_needed(lib_path: str,
                    no_dlls: set,
                    wheel_dirs: typing.Optional[typing.Iterable],
-                   on_error: str = 'raise') -> typing.Tuple[typing.Set[str], typing.Set[str], typing.Set[str]]:
+                   on_error: str,
+                   verbose: int) -> typing.Tuple[typing.Set[str], typing.Set[str], typing.Set[str]]:
     """Given the path to a shared library, return a 3-tuple of sets
     (discovered, ignored, not_found).
 
@@ -257,7 +275,7 @@ def get_all_needed(lib_path: str,
         lib_path = stack.pop()
         if lib_path not in discovered:
             discovered.add(lib_path)
-            with PEContext(lib_path) as pe:
+            with PEContext(lib_path, True, verbose) as pe:
                 imports = []
                 for attr in ('DIRECTORY_ENTRY_IMPORT', 'DIRECTORY_ENTRY_DELAY_IMPORT'):
                     if hasattr(pe, attr):
@@ -282,7 +300,7 @@ def get_all_needed(lib_path: str,
     return discovered, ignored, not_found
 
 
-def replace_needed(lib_path: str, old_deps: typing.Iterable, name_map: dict) -> None:
+def replace_needed(lib_path: str, old_deps: typing.Iterable, name_map: dict, verbose: int) -> None:
     """For the DLL at lib_path, replace its declared dependencies on old_deps
     with those in name_map.
     old_deps: a subset of the dependencies that lib_path has
@@ -311,6 +329,6 @@ def replace_needed(lib_path: str, old_deps: typing.Iterable, name_map: dict) ->
         raise ex
     with open(lib_path, 'wb') as f:
         f.write(buf)
-    with PEContext(lib_path) as pe:
+    with PEContext(lib_path, False, verbose) as pe:
         pe.OPTIONAL_HEADER.CheckSum = pe.generate_checksum()
         pe.write(lib_path)
